//--------------------------------------------------------------------------
//
// Copyright 2008, Cypress Semiconductor Corporation.
//
// This software is owned by Cypress Semiconductor Corporation (Cypress)
// and is protected by and subject to worldwide patent protection (United
// States and foreign), United States copyright laws and international
// treaty provisions. Cypress hereby grants to licensee a personal,
// non-exclusive, non-transferable license to copy, use, modify, create
// derivative works of, and compile the Cypress Source Code and derivative
// works for the sole purpose of creating custom software in support of
// licensee product to be used only in conjunction with a Cypress integrated
// circuit as specified in the applicable agreement. Any reproduction,
// modification, translation, compilation, or representation of this
// software except as specified above is prohibited without the express
// written permission of Cypress.
//
// Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND,EXPRESS OR IMPLIED,
// WITH REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
// Cypress reserves the right to make changes without further notice to the
// materials described herein. Cypress does not assume any liability arising
// out of the application or use of any product or circuit described herein.
// Cypress does not authorize its products for use as critical components in
// life-support systems where a malfunction or failure may reasonably be
// expected to result in significant injury to the user. The inclusion of
// Cypress' product in a life-support systems application implies that the
// manufacturer assumes all risk of such use and in doing so indemnifies
// Cypress against all charges.
//
// Use may be limited by and subject to the applicable Cypress software
// license agreement.
//
//*****************************************************************************
//*****************************************************************************
//  FILENAME: main.c
//   Version: 1.0, Updated on 27 August 2009
//
//  DESCRIPTION: Main file of the Example_RH_Sensor_HIH-4031 Project.
//
//-----------------------------------------------------------------------------
//  Copyright (c) Cypress MicroSystems 2000-2003. All Rights Reserved.
//*****************************************************************************
//*****************************************************************************
//*****************************************************************************
// 
//    The project can be tested using a Relative Humidity Sensor - HIH-4031.
// 
//CY3210-PSoCEVAL1 Board Project
//
//Project Objective
//	    To interface the Relative Humidity (RH) Sensor with PSoC and to display the 
//  humidity level on the LCD.
//
//Overview
//	This project is designed to demonstrate the interfacing of RH sensor with PSoC.
//	This RH sensor provides the analog voltage which is proportional to Humidity level,
//  this analog voltage is then measured using an ADC and the digital value is converted
//  to the humidity value and is displayed on the LCD.
//
//	The following changes were made to the default settings in the Device Editor:
// 
//    	Select User Modules
//    	    o Select ADCINC12_1, LCD_1, PGA_1 user module. 
//    	    o In this example, these UM is renamed as ADCINC12, LCD and PGA respectively. 
//    	Place User Modules
//    	    1. The User Module ADCINC12 occupies ASD11, DBB00, DBB01
//			2. The User Module PGA occupies ACB01
//			3. The User Module LCD is a software implementation, so it does not use any module.
//
//    	Set the global resources and UM parameters in the Device Editor as shown under 
//    	"Project Settings" ahead.
//
//
// Operation
//	Upon program execution all hardware settings from the device configuration are loaded 
//	into the device and main.c is executed. Analog voltage of the RH sensor is read by the 
//  ADCINC12 UM and is then converted to the digital value. This value is then converted to
//  the humidity value and is then displayed on the LCD.
//
// Circuit Connections
// 	This example can be tested using the CY3210 PSoC Eval1 board.  The following connections
// 	are to be made.  
// 
// 	PSoC
// 	--------
// 		P0[4]		 - Humidity Input
// 	
// 	HIH-4031 (3 Pin) - RH sensor		
// 	---------------------------
// 		Pin 1		- Ground    		
// 		Pin 2 		- Humidity Output	(Connected to P0[4])
// 		Pin 3		- Vcc
//
//Project Settings
//
//    Global Resources 
//		CPU_Clock      	= SysClk/2      		CPU clock set to 12 MHz
//		VC1			    = 8						Set VC1 at 3MHz for the operation of ADC
//		RefMux			= Vdd/2 +/- Vdd/2		Set the RefHi as 5V and RefLo as 0V
//
//User Module Parameters
//    ADCINC12		
//        TMR Clock   	= VC1	    			Set as 3MHz because SCBlock is referenced to this clock.
//        CNT Clock		= VC1		    	  	Set as 3MHz because SCBlock is referenced to this clock.
//        Input		    = ACB01					Selects the input as the output of the ACB01 block that 
//												is PGA configured as a Unity Gain Amplifier.
//        ClockPhase	= Norm 					Clock phase set to normal
//
//	  PGA		
//        Gain   		= 1.00	    					To make the PGA as a Unity Gain Amplifier.
//        Input			= AnalogColumn_InputSelect_1	Selects the output of Analog Column Input multiplexer 
//														as the input.
//        Reference		= Vss							Use the Vss of the IC as the reference.
//        AnalogBus		= Disable						Disable the connection to Analog Bus.
//
//	  LCD		
//        LCDPort  		= Port_2    			Use Port 2 for connecting LCD.
//        BarGraph		= Disable				Disable BarGraph feature.
//
//
//
//PGA
//    	Pin	    Select	            		Select			Drive				Interrupt
//    	----------------------------------------------------------------------------------
//    	P0[4]	AnalogColumn_InputMux_1 	AnalogInput		High Z Analog		DisableInt		
// 	
//----------------------------------------------------------------------------
// C main line
//----------------------------------------------------------------------------

#include <m8c.h>        // part specific constants and macros
#include "PSoCAPI.h"    // PSoC API definitions for all User Modules
#include "stdlib.h"		// Header file used to support standard library function link ftoa

// Calculate the humidity value based on the integer value provided
void Compute_Humidity(int iData_Humidity); 

void main(void)
{
	// Variable to hold the output of the ADC
	int iData;
	
	// Enable the Global interrupts. It is required because ADC uses interrupt for its operation. 
    M8C_EnableGInt;
	
	// Start the LCD.
	LCD_Start();

	// Start the PGA with the highest power setting.
	PGA_Start(PGA_HIGHPOWER);
	
	// Start the ADC with the highest power setting.
	ADCINC12_Start(ADCINC12_HIGHPOWER);
	
	// Run the ADC continuously.
	ADCINC12_GetSamples(0);
	
	// Position the LCD on row 0 and column 0 for printing a welcome message.
	LCD_Position(0,0);
	LCD_PrCString("Example Project ");

	// Infinite loop
	while(1)
	{
		// Wait till the valid data is available
		while(!ADCINC12_fIsDataAvailable());
		
		// Obtain the 2's compliment data and convert it to normal unsigned data by adding 0x800.
		iData = ADCINC12_iGetData() + 0x0800;

		// Compute the Humidty and display on the LCD.
		Compute_Humidity(iData);
	}
}

// This function is used to compute the humidity level based on the transfer function given for RH sensor.
// After calculating the humidity level, it updates the LCD with the same.
void Compute_Humidity(int iData_Humidity)
{
	// Variables used to convert the Hex Temperature data into Decimal format
	float fHumidity;
	char *floatRet;
	int iStatus;
	
	// Convert the digital value into its corresponding Analog value by multiplying it with one bit resolution.
	fHumidity = (float)iData_Humidity * 0.00122;
	
	LCD_Position(1,0);
	LCD_PrCString("RH Level: ");
	
	// Check if the humidity voltage is greater than zero voltage offset or not
	if(fHumidity >= 0.958)
	{
		// Compute theRH level using the transfer function provided in the datasheet.
		fHumidity = (fHumidity - 0.958)/0.0307;
		
		// Convert the float to Ascii in order to display it on the LCD
		floatRet = ftoa(fHumidity, &iStatus);
	
		// Display the RH level on the LCD.
		LCD_Position(1,10);
		LCD_PrString(floatRet);
		
		LCD_Position(1,15);
		LCD_PrCString("%");
	}
	else
	{
		// Display the RH level as 0% if o/p voltage is less than zero voltage offset.
		LCD_Position(1,10);
		LCD_PrCString("0.00% ");
	}
}